# parallelisme en python

![category](développement)
![tag](designpattern)
![tag](python)

Un petit exemple pour paralléliser des actions.
Dans notre exemple nous traitons de façon séquentielle une liste par une fonction qui prend 2sec (j'ai ajouté un time.sleep)

Notre liste prend 52sec pour être traité.
En répartissant le traitement sur 4 threads le temps est de 14sec ... un miracle

Nous utilisons pour cela une notion de thread et de file (queue)

```warning
en utilisant des files et des threads l'ordonnancement de l'entrée n'est pas celui de la sortie ... 
```

notre code

```python
import sys
import time
import threading, queue

class Action():

    def __init__(self, func, **kw):
        if not callable(func):
            raise ValueError("%s is not callable" % func)
        self.func = func
        self.kw = kw

    def run(self):
        return self.func(**self.kw)

class ParallelismThread(threading.Thread):

    def __init__(self, para):
        threading.Thread.__init__(self)
        self.para = para

    def run(self):
        while self.para.qin.qsize() > 0:
            act = self.para.qin.get()
            self.para.qout.put(act.run())

class Parallelism():

    def __init__(self, thread=1):
        self.threads = []
        for i in range(0, thread):
            self.threads.append(ParallelismThread(self))
        self.qin = queue.Queue()
        self.qout = queue.Queue()
    
    def append(self, func, **kw):
        self.qin.put(Action(func, **kw))
    
    def run(self):
        size = self.qin.qsize()
        for thread in self.threads:
            thread.start()
        while self.qout.qsize() < size:
            time.sleep(0.1)
        return [self.qout.get() for elt in range(0, size)]
        
    

## USAGE

def mytransform(char=""):
    time.sleep(2)
    return char.upper()


start_time = time.time()
listChars = 'abcdefghijklmnopqrstuvwxyz'
rst = ''
for char in listChars:
    rst = rst + mytransform(char=char)
print(rst)
print("Temps d execution : %s secondes ---" % (time.time() - start_time))


start_time = time.time()
parallelism = Parallelism(thread=4)
for char in listChars:
    parallelism.append(mytransform, char=char)
rst = parallelism.run()
print(''.join(rst))
print("Temps d execution : %s secondes ---" % (time.time() - start_time))
```
